﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace XamDataTree_Sample
{
    class DataUtil
    {
        private static ObservableCollection<Category> categories;
        private static ObservableCollection<Product> products;
        private static ObservableCollection<Category> categoriesAndProducts;

        private static ObservableCollection<Category> CreateCategories()
        {
            return new ObservableCollection<Category>
            {
                new Category 
                {
                    CategoryID = 1, 
                    CategoryName = "Beverages"
                },

                new Category 
                {
                    CategoryID = 2, 
                    CategoryName = "Condiments"
                },
                
                new Category
                {
                    CategoryID = 3,
                    CategoryName = "Confections"
                }
            };
        }

        private static ObservableCollection<Product> CreateProducts()
        {
            return new ObservableCollection<Product>
            {
                new Product
                {
                    ProductID = 1,
                    ProductName = "Chai",
                    CategoryID = 1,
                    Tests = new ObservableCollection<Test>
                    {
                        new Test
                        {
                            TestID = 1,
                            TestName= "Test1"
                        }
                    }
                },

                new Product
                {
                    ProductID = 2,
                    ProductName = "Chang",
                    CategoryID = 1,
                    Tests = new ObservableCollection<Test>
                    {
                        new Test
                        {
                            TestID = 2,
                            TestName= "Test2"
                        }
                    }
                },

                new Product
                {
                    ProductID = 24,
                    ProductName = "Guarana Fantastica",
                    CategoryID = 1
                },

                new Product
                {
                    ProductID = 77,
                    ProductName = "Original Frankfurter grune Sose",
                    CategoryID = 2
                },

                new Product
                {
                    ProductID = 61,
                    ProductName = "Sirop d'erable",
                    CategoryID = 2
                },

                new Product
                {
                    ProductID = 63,
                    ProductName = "Vegie-spread",
                    CategoryID = 2
                },

                new Product
                {
                    ProductID = 16,
                    ProductName = "Pavlova",
                    CategoryID = 3
                },

                new Product
                {
                    ProductID = 19,
                    ProductName = "Teatime Chocolate Biscuits",
                    CategoryID = 3
                },

                new Product
                {
                    ProductID = 20,
                    ProductName = "Sir Rodney's Marmalade",
                    CategoryID = 3
                }
            };
        }

        private static ObservableCollection<Category> CreateCategoriesAndProducts()
        {
            ObservableCollection<Category> retVal = CreateCategories();

            ObservableCollection<Product> tempProducts = CreateProducts();

            Category parentCategory = null;

            foreach (Product p in tempProducts)
            {
                if (parentCategory == null || parentCategory.CategoryID != p.CategoryID)
                {
                    parentCategory = retVal.First<Category>(category => category.CategoryID == p.CategoryID);
                }

                parentCategory.Products.Add(p);
            }

            return retVal;
        }

        public static ObservableCollection<Category> Categories
        {
            get
            {
                if (categories == null)
                    categories = CreateCategories();

                return categories;
            }
        }

        public static ObservableCollection<Product> Products
        {
            get
            {
                if (products == null)
                    products = CreateProducts();

                return products;
            }
        }

        public static ObservableCollection<Category> CategoriesAndProducts
        {
            get
            {
                if (categoriesAndProducts == null)
                    categoriesAndProducts = CreateCategoriesAndProducts();

                return categoriesAndProducts;
            }
        }
    }

    public class Category : INotifyPropertyChanged
    {
        private int categoryID;
        private string categoryName;
        private ObservableCollection<Product> products;

        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                    OnPropertyChanged("CategoryID");
                }
            }
        }

        public string CategoryName
        {
            get
            {
                return this.categoryName;
            }
            set
            {
                if (this.categoryName != value)
                {
                    this.categoryName = value;
                    OnPropertyChanged("CategoryName");
                }
            }
        }

        public ObservableCollection<Product> Products
        {
            get
            {
                if (products == null)
                {
                    products = new ObservableCollection<Product>();
                }

                return this.products;
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
    }

    public class Product : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private int productID;
        private string productName;
        private int categoryID;
        private ObservableCollection<Test> tests;

        public int ProductID
        {
            get
            {
                return this.productID;
            }
            set
            {
                if (this.productID != value)
                {
                    this.productID = value;
                    OnPropertyChanged("ProductID");
                }
            }
        }

        public string ProductName
        {
            get
            {
                return this.productName;
            }
            set
            {
                if (this.productName != value)
                {
                    this.productName = value;
                    OnPropertyChanged("ProductName");
                }
            }
        }

        public int CategoryID
        {
            get
            {
                return this.categoryID;
            }
            set
            {
                if (this.categoryID != value)
                {
                    this.categoryID = value;
                    OnPropertyChanged("CategoryID");
                }
            }
        }

        public ObservableCollection<Test> Tests
        {
            get
            {
                if (tests == null)
                {
                    tests = new ObservableCollection<Test>();
                }

                return this.tests;
            }
            set
            {
                if (this.tests != value)
                {
                    this.tests = value;
                    OnPropertyChanged("Tests");
                }
            }
        }
    }


    public class Test : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private int testID;
        private string testName;
        private int productID;

        public string TestName
        {
            get
            {
                return this.testName;
            }
            set
            {
                if (this.testName != value)
                {
                    this.testName = value;
                    OnPropertyChanged("TestName");
                }
            }
        }

        public int TestID
        {
            get
            {
                return this.testID;
            }
            set
            {
                if (this.testID != value)
                {
                    this.testID = value;
                    OnPropertyChanged("TestID");
                }
            }
        }

        public int ProductID
        {
            get
            {
                return this.productID;
            }
            set
            {
                if (this.productID != value)
                {
                    this.productID = value;
                    OnPropertyChanged("ProductID");
                }
            }
        }
    }
}